/***************************************************************************
 * e2pwd - Prints current directory
 *
 * Copyright (C) 1995 Claus Tondering, ct@login.dknet.dk
 * Copyright (C) 1997 Peeter Joot
 *
 * This file may be redistributed under the terms of the GNU Public License.
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <linux/types.h>

#include "linux/ext2_fs.h"
#include "ext2fs/ext2fs.h"
#include "linux/stat.h"
#include "ext2fs/ext2_err.h"
#include "dev.h"
#include "e2err.h"

extern io_manager nt_io_manager;

int aflag, dflag, iflag, lflag, tflag, rflag;

struct fileinfo {
	ino_t	inode;
	unsigned char	name[80];
	struct ext2_inode e2ino;
} *list;
int listix;
int maxlist;

char directory[256];
char dir2[256];

ino_t ino;
char * prog_name;

/**********************************************************************
 * myproc is a callback routine which is called once for each entry
 * in the directory
 **********************************************************************/

static int myproc(struct ext2_dir_entry *dirent,
				  int	offset,
				  int	blocksize,
				  char	*buf,
				  void	*private)
{
	if (dirent->inode == ino) {
		if (directory[0]==0) {
			strncpy(directory, dirent->name, dirent->name_len);
			directory[dirent->name_len]=0;
		}
		else {
			strncpy(dir2, dirent->name, dirent->name_len);
			dir2[dirent->name_len]=0;
			strcat(dir2, "/");
			strcat(dir2,directory);
			strcpy(directory,dir2);
		}
		return DIRENT_ABORT;
	}

	return 0;
}


/**********************************************************************
 * usage prints usage information and exits
 **********************************************************************/

void
usage()
{
	fprintf(stderr, 
"SYNOPSIS\n"
"\tE2PWD [-h]\n"
"\n"
"DESCRIPTION\n"
"\tThe E2PWD program prints the current directory.\n"
"\n"
"-h\tPrints this message.\n"
);

	exit(1);
}


/**********************************************************************
 * main routine
 **********************************************************************/

int main(int argc, char **argv)
{
	int err;
	ext2_filsys fs;
	ino_t dotdotino;
	struct ext2_inode e2ino;
	e2cwd_t cwd;

	prog_name = argv[0];

	if (argc!=1)
		usage();

	initialize_ext2_error_table();

        err = cwdenv_lookup(&cwd);
        if (err)
                com_err("e2cat",0,"E2CWD environment has bad format.",0);

	/* Open file system */
	err = ext2fs_open((const char *)&cwd, 0, 0, 0, nt_io_manager, &fs);
	if (err)
		e2_err(EXT2_ET_CANT_OPEN_FS,err);

	/* Loop until we have the root */
	for (ino=cwd.cwdino; ino!=2; ino=dotdotino) {
		/* Look up '..' */
		err = ext2fs_namei(fs, 2, ino, "..", &dotdotino);
		if (err)
			e2_err(EXT2_ET_CANT_FIND_DOTDOT,err);

		err = ext2fs_read_inode(fs, dotdotino, &e2ino);
		if (err)
			e2_err(EXT2_ET_CANT_READ_DOTDOT,err);

		if (!S_ISDIR(e2ino.i_mode)) {
			fprintf(stderr,".. is not a directory\n");
			exit(1);
		}

		/* Loop through all directory entries */
		err = ext2fs_dir_iterate(fs, dotdotino, 0, 0, myproc, 0);
		if (err)
			e2_err(EXT2_ET_CANT_READ_DIR,err);
	}

	printf("/%s\n",directory);

	return 0;
}		
